/* Copyright (c) 1997, MpegTV LLC */
#ifndef _msg_h
#define _msg_h
#ifndef lint
static const char rcsid_msg_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/msg.h,v 1.28 1999/09/08 07:13:52 tristan Exp tristan $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "misc.h"
#include "ceutils.h"

/*--------------------------------------------------------------------------+
| Configuration																|
+--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
#ifndef MSG_DISABLED

#ifndef WIN32
#define _msg_init()
#define _msg_puts(str)				fputs(str, stdout)
#define _msg_printf	printf
#define _MSG_END_OF_LINE	"\n"
#define output_debug_string(str)	_msg_puts(str)
#else
#define _MSG_END_OF_LINE	"\r\n"
#ifdef ARM
/* work around compiler bug */
#define _msg_puts(str)				_msg_putss(str)
#endif
#endif

#endif

/*
 * Print message if specified message type is selected in MTVP_MSG_PTR->select
 */
#ifdef MSG_DISABLED
#define MTVP_MSG(type, message)
#else
#define MTVP_MSG(type, message) STMT( \
	if (MTVP_MSG_PTR->select & (type)) { \
		_msg_header(MTVP_MSG_PTR, MTVP_MSG_HEADER_INFO, __FILE__, __LINE__); \
		_msg_printf message; \
		_msg_puts(_MSG_END_OF_LINE); \
	} \
)
#endif

/*
 * Print message and error diagnotic if specified message type is selected in MTVP_MSG_PTR->select
 */
#ifdef MSG_DISABLED
#define MTVP_MSG_ERROR(type, message)
#else
#define MTVP_MSG_ERROR(type, message) STMT( \
	if (MTVP_MSG_PTR->select & (MTVP_MSG_ERR|(type))) { \
		int	_msg_status = status; \
		_msg_header(MTVP_MSG_PTR, MTVP_MSG_HEADER_ERROR, __FILE__, __LINE__); \
		_msg_printf message; \
		_msg_printf(": %s (status=%d)", msg_error_string(_msg_status), _msg_status); \
		_msg_puts(_MSG_END_OF_LINE); \
	} \
)
#endif

/*
 * Print info message
 */
#define MTVP_MESSAGE(message) 	MTVP_MSG(MTVP_MSG_INFO, message)

/*
 * Print error message
 */
#define MTVP_MESSAGE_ERROR(message) 	MTVP_MSG_ERROR(MTVP_MSG_ERR, message)

/*
 * Print warning message
 */
#ifdef MSG_DISABLED
#define MTVP_WARNING(printf_param)
#else
#define MTVP_WARNING(printf_param)	STMT( \
		if (MTVP_MSG_PTR->select & MTVP_MSG_WARNING) { \
			_msg_header(MTVP_MSG_PTR, MTVP_MSG_HEADER_WARNING, __FILE__, __LINE__); \
			_msg_printf printf_param; \
			_msg_puts(_MSG_END_OF_LINE); \
		} \
	)
#endif


/*
 * Print message then exit
 */
#ifdef MSG_DISABLED
#define MTVP_FATAL(printf_param)	STMT( \
		exit(EXIT_FAILURE); \
	)
#else
#define MTVP_FATAL(printf_param)	STMT( \
		_msg_header(MTVP_MSG_PTR, MTVP_MSG_HEADER_FATAL, __FILE__, __LINE__); \
		_msg_printf printf_param; \
		_msg_puts(_MSG_END_OF_LINE); \
		exit(EXIT_FAILURE); \
	)
#endif

/*
 * Print message and error diagnotic then exit
 */
#ifdef MSG_DISABLED
//#define FATAL_ERROR(printf_param)	STMT( \
//		_msg_printf("fatal: %s (errno=%d)\n", msg_error_string(errno), errno); \
//		exit(EXIT_FAILURE); \
//	)
#define MTVP_FATAL_ERROR(printf_param)	STMT( \
		exit(EXIT_FAILURE); \
	)
#else
#define MTVP_FATAL_ERROR(printf_param)	STMT( \
		int	_msg_status = status; \
		_msg_header(MTVP_MSG_PTR, MTVP_MSG_HEADER_FATAL, __FILE__, __LINE__); \
		_msg_printf printf_param; \
		_msg_printf(": %s (status=%d)", msg_error_string(_msg_status), _msg_status); \
		_msg_puts(_MSG_END_OF_LINE); \
		exit(EXIT_FAILURE); \
	)
#endif

/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/
/* for _msg_header */
typedef enum {
	MTVP_MSG_HEADER_INFO = 0,	/* for MESSAGE() */
	MTVP_MSG_HEADER_WARNING = 1,	/* for WARNING() */
	MTVP_MSG_HEADER_ERROR = 2,	/* for ERROR() i.e. non-fatal errors */
	MTVP_MSG_HEADER_FATAL = 3	/* for FATAL() i.e. fatal errors */
} MTVP_MSG_HEADER;

/*
 * Masks used in MTVP_MTVP_MSG_PTR->select to select which type of messages should be printed.
 *
 * MTVP_MTVP_MSG_PTR->select is a bitwise OR of those flags, and possibly more flags
 * defined by the application.
 */

#define	MTVP_MSG_INFO		(1 << 0)	/* for MESSAGE() */
#define	MTVP_MSG_ERR			(1 << 1)	/* for ERROR() i.e. non-fatal errors */
#define	MTVP_MSG_WARNING		(1 << 2)	/* for WARNING() */

typedef struct msg_struct {
	u_long select;
	const char *progname;
	void (*header_func)(const struct msg_struct *msg_ptr, MTVP_MSG_HEADER msg_header_type, const char *filename, int line, TCHAR *buffer, u_int size);
} MTVP_MSG_STRUCT;

/*--------------------------------------------------------------------------+
| Externals																	|
+--------------------------------------------------------------------------*/
extern const MTVP_MSG_STRUCT Msg_dummy;

#ifdef __cplusplus
extern "C" {
#endif

#ifdef WIN32

#if defined(UNDER_CE) && defined(ARM)
/* work around compiler bug ? */
#define _msg_puts _msg_putss
#endif

extern void _msg_init(void);

extern void _msg_puts(const char *str);
extern void _msg_printf(const char *format, ...);
#endif

#ifndef MSG_DISABLED

extern const char * _msg_basename(const char *);
extern void _msg_header(const MTVP_MSG_STRUCT *msg_ptr, MTVP_MSG_HEADER msg_header_type, const char *filename, int line);

#endif

const char *msg_error_string(int code);

#ifdef __cplusplus
}
#endif


#endif /* _msg_h */
