/* Copyright (c) 1997, MpegTV LLC */
#ifndef _mtvp_player_hpp
#define _mtvp_player_hpp
#ifndef lint
static const char rcsid_mtvp_player_hpp[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/g2a_api.h,v 1.1 1999/08/03 08:05:19 tristan Exp tristan $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/mtvp.h"


/*--------------------------------------------------------------------------+
| External
+--------------------------------------------------------------------------*/
/* construction zone! this class is not fully defined yet */
class MTVP_Player {
private:
	int	synchronous_flag;
public:
	MTVP_PLAYER *player;
	int	voh_handler_id;

	MTVP_Player() {
		player = (MTVP_PLAYER *)0;
		synchronous_flag = 0;
	}
	~MTVP_Player() {
		if (player) {
			mtvp_destroy_player(player);
		}
	}
	int Run(const char **mtvp_argv, const char *argv0, void *platform_ptr) {
		int status;
		status = mtvp_create_player(&player, mtvp_argv, argv0, platform_ptr);
		if (status) player = (MTVP_PLAYER *)0;
		return status;
	}
	void HandleNotificationMessage(MTVP_PMP_HEADER *msg, const MTVP_PMP_DATA_A2G *data) {

		switch (msg->type) {
			case MTVP_NOTIFY_RETURN_SUCCESS:
				OnNotifySuccess();
				break;
			case MTVP_NOTIFY_RETURN_FAILURE:
				OnNotifyFailure();
				break;
			case MTVP_NOTIFY_READY:
				OnNotifyReady();
				break;
			case MTVP_NOTIFY_QUIT:
				OnNotifyQuit();
				break;
			case MTVP_NOTIFY_STATE:
				OnNotifyState((MTVP_STATE)data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_POS:
				OnNotifyPosition(data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_LOOP_END:
				OnNotifyLoopEnd();
				break;
			case MTVP_NOTIFY_ACTUAL_FRAMERATE:
				OnNotifyActualFramerate(data->data_generic.data_double);
				break;
			case MTVP_NOTIFY_AUDIO_ENABLED:
				OnNotifyAudioEnabled();
				break;
			case MTVP_NOTIFY_AUDIO_DISABLED:
				OnNotifyAudioDisabled();
				break;
			case MTVP_NOTIFY_AUDIO_VOLUME:
				OnNotifyVolume(data->data_generic.data_double);
				break;
			case MTVP_NOTIFY_AUDIO_BALANCE:
				OnNotifyBalance(data->data_generic.data_double);
				break;
			case MTVP_NOTIFY_AUDIO_MUTE:
				OnNotifyMute();
				break;
			case MTVP_NOTIFY_VIDEO_DITHER_TYPE:
				OnNotifyDitherType((MTVP_DITHER_TYPE)data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_VIDEO_DITHER_FLAGS:
				OnNotifyDitherFlags(data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_INPUT_FILE_SIZE:
				OnNotifyFileSize((u_int)data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_DISPLAY_READY:
				OnNotifyDisplayReady();
				break;
			case MTVP_NOTIFY_VIDEO_INFOS:
				OnNotifyVideoInfos((const MTVP_PMP_DATA_VIDEO_INFOS *)data);
				break;
			case MTVP_NOTIFY_GOP_INFOS:
				OnNotifyGOPInfos((const MTVP_PMP_DATA_GOP_INFOS *)data);
				break;
			case MTVP_NOTIFY_AUDIO_INFOS:
				OnNotifyAudioInfos((const MTVP_PMP_DATA_AUDIO_INFOS *)data);
				break;
			case MTVP_NOTIFY_FRAME_INFOS:
				OnNotifyFrameInfos((const MTVP_PMP_DATA_FRAME_INFOS *)data);
				break;
			case MTVP_NOTIFY_PACK_INFOS:
				OnNotifyPackInfos((const MTVP_PMP_DATA_PACK_INFOS *)data);
				break;
			case MTVP_NOTIFY_VOH_FEEDBACK:
				OnNotifyVOHFeedback((const MTVP_VOH_FEEDBACK *)data, msg->size);
				break;
			case MTVP_NOTIFY_SIH_FEEDBACK:
				OnNotifySIHFeedback((const MTVP_SIH_FEEDBACK *)data, msg->size);
				break;
			case MTVP_NOTIFY_MPEG2_VIDEO:
				OnNotifyMpeg2Video();
				break;
			case MTVP_NOTIFY_BAD_STREAM:
				OnNotifyBadStream();
				break;
			case MTVP_NOTIFY_CANNOT_PLAY_AUDIO:
				OnNotifyCannotPlayAudio(data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_CANNOT_SHOW_VIDEO:
				OnNotifyCannotShowVideo(data->data_generic.data_int);
				break;
		}
	}
	void SetSynchronousMode() {
		synchronous_flag = MTVP_PMP_FLAG_SYNCHRONOUS;
	}
	int SendCommandCustom(MTVP_COMMAND command, const void *ptr, u_int size) {
		return mtvp_send_command_custom(player, command, synchronous_flag, ptr, size);
	}
private:
	int SendCommandVoid(MTVP_COMMAND command) {
		return mtvp_send_command_void(player, command, synchronous_flag);
	}
	int SendCommandInt(MTVP_COMMAND command, int data) {
		return mtvp_send_command_int(player, command, synchronous_flag, data);
	}
	int SendCommandString(MTVP_COMMAND command, const char *string) {
		return mtvp_send_command_string(player, command, synchronous_flag, string);
	}
	int SendCommandDouble(MTVP_COMMAND command, double d) {
		return mtvp_send_command_double(player, command, synchronous_flag, d);
	}
public:
	int GetCommandStatus(int *status, void **value) {
		int ret = mtvp_get_command_status(player, status, value);
		synchronous_flag = 0;
		return ret;
	}
	int OpenStreamURL(const char *url) {
		return SendCommandString(MTVP_COMMAND_OPEN_STREAM_URL, url);
	}
	int CloseStream() {
		return SendCommandVoid(MTVP_COMMAND_CLOSE_STREAM);
	}
	int SetState(MTVP_STATE state) {
		return SendCommandInt(MTVP_COMMAND_STATE, state);
	}
	int Play() {
		return SetState(MTVP_STATE_PLAYING);
	}
	int Pause() {
		return SetState(MTVP_STATE_PAUSED);
	}
	int Steps(u_int n) {
		return SendCommandInt(MTVP_COMMAND_STEPS, n);
	}
	int Seek(u_int pos) {
		return SendCommandInt(MTVP_COMMAND_SEEK, (u_int)pos);
	}
	int SeekToEnd() {
		return SendCommandVoid(MTVP_COMMAND_END);
	}
	int SetAudioChannel(u_int channel) {
		return SendCommandInt(MTVP_COMMAND_AUDIO_CHANNEL, channel);
	}
	int SetAudioQuality(u_int quality) {
		return SendCommandInt(MTVP_COMMAND_AUDIO_QUALITY, quality);
	}
	int Loop(u_int yes) {
		return SendCommandInt(MTVP_COMMAND_LOOP, yes);
	}
	int AutoPlay(u_int yes) {
		return SendCommandInt(MTVP_COMMAND_AUTO_PLAY, yes);
	}
	int Mute(u_int yes) {
		return SendCommandInt(MTVP_COMMAND_AUDIO_MUTE, yes);
	}
	int Repaint() {
		return SendCommandVoid(MTVP_COMMAND_VIDEO_REPAINT);
	}
	int Focus() {
		return SendCommandVoid(MTVP_COMMAND_VIDEO_FOCUS);
	}
	int Quit() {
		return SendCommandVoid(MTVP_COMMAND_QUIT);
	}
	int SetDisplayWindow(int window_id) {
		return SendCommandInt(MTVP_COMMAND_WINDOW_ID, window_id);
	}
	int SetGammaCorrection(double x) {
		return SendCommandDouble(MTVP_COMMAND_VIDEO_GAMMA, x);
	}
	int SetChromaSaturation(double x) {
		return SendCommandDouble(MTVP_COMMAND_VIDEO_CHROMA_SATURATION, x);
	}
	int SetSyncType(MTVP_SYNC_TYPE sync_type) {
		return SendCommandInt(MTVP_COMMAND_SYNC_TYPE, (int)sync_type);
	}
//	int SetFramerate(double rate) {
//		return SendCommandDouble(MTVP_COMMAND_SYNC_FRAMERATE, rate);
//	}
	int SetRateFactor(double rate) {
		return SendCommandDouble(MTVP_COMMAND_SYNC_RATE_FACTOR, rate);
	}
	int SetVolume(double volume) {
		return SendCommandDouble(MTVP_COMMAND_AUDIO_VOLUME, volume);
	}
	int SetBalance(double balance) {
		return SendCommandDouble(MTVP_COMMAND_AUDIO_BALANCE, balance);
	}
	int SetAudioControls(MTVP_AUDIO_CONTROLS n) {
		return SendCommandInt(MTVP_COMMAND_AUDIO_CONTROLS, (int)n);
	}
	int LinkSIH(const char *path) {
		return SendCommandString(MTVP_COMMAND_SIH_LINK, path);
	}
	int InitSIH(int handler_id) {
		return SendCommandInt(MTVP_COMMAND_SIH_INIT, handler_id);
	}
	int FreeSIH(int handler_id) {
		return SendCommandInt(MTVP_COMMAND_SIH_FREE, handler_id);
	}
	int ControlSIH(const MTVP_SIH_CNTL *data, u_int size) {
		return SendCommandCustom(MTVP_COMMAND_SIH_CNTL, data, size);
	}
	int LinkVOH(const char *path) {
		return SendCommandString(MTVP_COMMAND_VOH_LINK, path);
	}
	int InitVOH(int handler_id) {
		return SendCommandInt(MTVP_COMMAND_VOH_INIT, handler_id);
	}
	int FreeVOH(int handler_id) {
		return SendCommandInt(MTVP_COMMAND_VOH_FREE, handler_id);
	}
	int SelectVOH(int handler_id) {
		return SendCommandInt(MTVP_COMMAND_VOH_SELECT, handler_id);
	}
	int ControlVOH(const MTVP_VOH_CNTL *data, u_int size) {
		return SendCommandCustom(MTVP_COMMAND_VOH_CNTL, data, size);
	}
	int GetPosition() {
		return SendCommandVoid(MTVP_COMMAND_GET_POS);
	}
	int GetActualFramerate() {
		return SendCommandVoid(MTVP_COMMAND_GET_ACTUAL_FRAMERATE);
	}
	int GetNumberOfFramesPlayed() {
		return SendCommandVoid(MTVP_COMMAND_GET_NUMBER_OF_FRAMES_PLAYED);
	}
	int MsgSelect(int msg_mask) {
		return SendCommandInt(MTVP_COMMAND_MSG_SELECT, msg_mask);
	}
protected:
	virtual void OnNotifySuccess() {}
	virtual void OnNotifyFailure() {}
	virtual void OnNotifyReady() {}
	virtual void OnNotifyQuit() {}
	virtual void OnNotifyState(MTVP_STATE state) {}
	virtual void OnNotifyPosition(int pos) {}
	virtual void OnNotifyLoopEnd() {}
	virtual void OnNotifyActualFramerate(double x) {}
	virtual void OnNotifyAudioEnabled() {}
	virtual void OnNotifyAudioDisabled() {}
	virtual void OnNotifyVolume(double x) {}
	virtual void OnNotifyBalance(double x) {}
	virtual void OnNotifyMute() {}
	virtual void OnNotifyDitherType(MTVP_DITHER_TYPE type) {}
	virtual void OnNotifyDitherFlags(int flags) {}
	virtual void OnNotifyFileSize(u_int size) {}
	virtual void OnNotifyDisplayReady() {}
	virtual void OnNotifyVideoInfos(const MTVP_PMP_DATA_VIDEO_INFOS *ptr) {}
	virtual void OnNotifyGOPInfos(const MTVP_PMP_DATA_GOP_INFOS *ptr) {}
	virtual void OnNotifyAudioInfos(const MTVP_PMP_DATA_AUDIO_INFOS *ptr) {}
	virtual void OnNotifyFrameInfos(const MTVP_PMP_DATA_FRAME_INFOS *ptr) {}
	virtual void OnNotifyPackInfos(const MTVP_PMP_DATA_PACK_INFOS *ptr) {}
	virtual void OnNotifyVOHFeedback(const MTVP_VOH_FEEDBACK *data, u_int size) {}
	virtual void OnNotifySIHFeedback(const MTVP_SIH_FEEDBACK *data, u_int size) {}
	virtual void OnNotifyMpeg2Video() {}
	virtual void OnNotifyBadStream() {}
	virtual void OnNotifyCannotPlayAudio(int status) {}
	virtual void OnNotifyCannotShowVideo(int status) {}
};



#endif /* _mtvp_player_hpp */
