// PocketTV
//

#ifndef STRICT
#define STRICT
#endif

#include "stdafx.h"
#include "pockettv.hpp"

#include <commctrl.h>
#include <commdlg.h>
#include <winnls.h>
#ifdef HAVE_AYGSHELL
#include <aygshell.h>
#endif
#include <stdlib.h>

#include "files.h"

#define IDM_OPTIONS_AUDIO_QUALITY_FIRST Min(IDM_OPTIONS_AUDIO_QUALITY_HIGH, Min(IDM_OPTIONS_AUDIO_QUALITY_LOW, IDM_OPTIONS_AUDIO_QUALITY_MEDIUM))
#define IDM_OPTIONS_AUDIO_QUALITY_LAST Max(IDM_OPTIONS_AUDIO_QUALITY_HIGH, Max(IDM_OPTIONS_AUDIO_QUALITY_LOW, IDM_OPTIONS_AUDIO_QUALITY_MEDIUM))
#define IDM_OPTIONS_AUDIO_CHANNEL_FIRST Min(IDM_OPTIONS_AUDIO_CHANNEL_BOTH, Min(IDM_OPTIONS_AUDIO_CHANNEL_LEFT, IDM_OPTIONS_AUDIO_CHANNEL_RIGHT))
#define IDM_OPTIONS_AUDIO_CHANNEL_LAST Max(IDM_OPTIONS_AUDIO_CHANNEL_BOTH, Max(IDM_OPTIONS_AUDIO_CHANNEL_LEFT, IDM_OPTIONS_AUDIO_CHANNEL_RIGHT))
#define IDM_OPTIONS_VIDEO_STRETCH_FIRST Min(IDM_OPTIONS_VIDEO_STRETCH_FILL, Min(IDM_OPTIONS_VIDEO_STRETCH_AUTO, Min(IDM_OPTIONS_VIDEO_STRETCH_NEVER, IDM_OPTIONS_VIDEO_STRETCH_ALWAYS)))
#define IDM_OPTIONS_VIDEO_STRETCH_LAST Max(IDM_OPTIONS_VIDEO_STRETCH_FILL, Max(IDM_OPTIONS_VIDEO_STRETCH_AUTO, Max(IDM_OPTIONS_VIDEO_STRETCH_NEVER, IDM_OPTIONS_VIDEO_STRETCH_ALWAYS)))

#define TIMER_POSITION_INTERVAL		1000
#define TIMER_FRAMERATE_INTERVAL	5000
#define TIMER_KEEPALIVE_INTERVAL	30000	// to prevent auto-shutdown while playing. must be less than 1 minute!
#define VOLUME_SLIDER_MAX			100
#define TRACK_SLIDER_MAX			10000
#define FAST_FORWARD_COEF			8.0


/*--------------------------------------------------------------------------+
| MyLoadString
+--------------------------------------------------------------------------*/
const TCHAR *
MyLoadString(HINSTANCE instance, int id)
{
	const TCHAR *p = (const TCHAR *)LoadString(instance, id, 0, 0);
	if (p == NULL) {
		return NULL;
	}
	u_int n = *((WORD *)p - 1);
	const TCHAR *q = (const TCHAR *)memory_calloc(n + 1, sizeof(TCHAR));
	if (q == NULL) {
		return NULL;
	}
	LoadString(instance, id, q, n + 1);
	return q;
}

/*----------------------------------------------------------------------
|       PocketTVWindowProc
+---------------------------------------------------------------------*/
LRESULT CALLBACK
PocketTVWindowProc(HWND window, UINT message, WPARAM wparam, LPARAM lparam)
{
	POCKETTV_HANDLE_WINDOW_MESSAGE(PocketTV, 
                             HandleWindowsMessage,
                             window, 
                             message, 
                             wparam, 
                             lparam);
}

/*----------------------------------------------------------------------
| center_dialog
+---------------------------------------------------------------------*/
static void
center_dialog(HWND hDlg) {
	RECT rt, rt1;
	int DlgWidth, DlgHeight;	// dialog width and height in pixel units
	int NewPosX, NewPosY;

	// trying to center the dialog window
	if (GetWindowRect(hDlg, &rt1)) {
		GetClientRect(GetParent(hDlg), &rt);
		DlgWidth	= rt1.right - rt1.left;
		DlgHeight	= rt1.bottom - rt1.top ;
		NewPosX		= (rt.right - rt.left - DlgWidth)/2;
		NewPosY		= (rt.bottom - rt.top - DlgHeight)/2;
		
		// if the About box is larger than the physical screen 
		if (NewPosX < 0) NewPosX = 0;
		if (NewPosY < 0) NewPosY = 0;
		SetWindowPos(hDlg, 0, NewPosX, NewPosY,
			0, 0, SWP_NOZORDER | SWP_NOSIZE);
	}
}

#ifdef SPI_SETSIPINFO

/*----------------------------------------------------------------------
| LowerSip
+---------------------------------------------------------------------*/
static BOOL
LowerSip( void )
{   
    BOOL fRes = FALSE;
    SIPINFO si;
    memset( &si, 0, sizeof( si ) );
    si.cbSize = sizeof( si );
    if( SHSipInfo( SPI_GETSIPINFO, 0, &si, 0 ) )
    {
        si.fdwFlags &= ~SIPF_ON;
        fRes = SHSipInfo( SPI_SETSIPINFO, 0, &si, 0 );
    }    
    return fRes;
}

/*----------------------------------------------------------------------
| RaiseSip
+---------------------------------------------------------------------*/
static BOOL
RaiseSip( void )
{   
    BOOL fRes = FALSE;
    SIPINFO si;
    memset( &si, 0, sizeof( si ) );
    si.cbSize = sizeof( si );
    if( SHSipInfo( SPI_GETSIPINFO, 0, &si, 0 ) )
    {
        si.fdwFlags |= SIPF_ON;
        fRes = SHSipInfo( SPI_SETSIPINFO, 0, &si, 0 );
    }    
    return fRes;
}

#else

#define LowerSip()	TRUE
#define RaiseSip()	TRUE

#endif

/*----------------------------------------------------------------------
| DialogAbout
+---------------------------------------------------------------------*/
LRESULT CALLBACK 
DialogAbout(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	// Mesage handler for the About box.

	switch (message)
	{
		case WM_INITDIALOG:
			center_dialog(hDlg);
			return TRUE;

		case WM_COMMAND:
			if ((LOWORD(wParam) == IDOK) || (LOWORD(wParam) == IDCANCEL))
			{
				EndDialog(hDlg, LOWORD(wParam));
				return TRUE;
			}
			break;
	}
    return FALSE;
}


/*--------------------------------------------------------------------------+
| ErrorBox
+--------------------------------------------------------------------------*/
void
PocketTV::ErrorBox(HWND window, const TCHAR *format, ...)
{
	va_list	args;
	TCHAR	buffer[512];
	TCHAR *title = MyLoadString(m_Instance, IDS_WINDOW_CAPTION_ERROR);

	va_start(args, format);
	vswprintf(buffer, format, args);
//	_vsnwprintf(buffer, sizeofarray(buffer) - 1, format, args)

	va_end(args);
	buffer[sizeofarray(buffer) - 1] = 0;

	MessageBox(window, buffer, title, MB_OK | MB_TOPMOST | MB_APPLMODAL);
	memory_free(title);
}


/*--------------------------------------------------------------------------+
| get_msg_error_string
+--------------------------------------------------------------------------*/
TCHAR *
get_msg_error_string(int status)
{
	LPTSTR lpMsgBuf = NULL;
	USES_CONVERSION;
	int	ret;

	if (status > 0) {
		ret = FormatMessage( 
			FORMAT_MESSAGE_ALLOCATE_BUFFER | 
			FORMAT_MESSAGE_FROM_SYSTEM,
			NULL,
			status,
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
			(LPTSTR) &lpMsgBuf,
			0,
			NULL );
		return lpMsgBuf;
	} else {
		return _wcsdup(A2W(msg_error_string(status)));
	}
}


/*----------------------------------------------------------------------
| DialogOpenURL
+---------------------------------------------------------------------*/
LRESULT CALLBACK 
DialogOpenURL(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message)
	{
		case WM_INITDIALOG:
			RaiseSip();
			center_dialog(hDlg);
			{
				PocketTV *player;
				u_int	n;
				TCHAR	*url;
				USES_CONVERSION;

				player = (PocketTV *)GetWindowLong(GetParent(hDlg), GWL_USERDATA);
				url = player->m_URLName ? A2W(player->m_URLName) : TEXT("http://");
				SetDlgItemText(hDlg, IDC_EDIT_URL, url);
				n = wcslen(url);
				SendMessage(GetDlgItem(hDlg, IDC_EDIT_URL), EM_SETSEL, n, n);
			}
			SetFocus(GetDlgItem(hDlg, IDC_EDIT_URL));
			return FALSE;
		case WM_COMMAND:
			if (LOWORD(wParam) == IDOK) {
				TCHAR url[256];
				PocketTV *player;
				int status;

				player = (PocketTV *)GetWindowLong(GetParent(hDlg), GWL_USERDATA);

				
				url[sizeofarray(url) - 1] = 0;
				if (GetDlgItemText(
						hDlg,       // handle of dialog box
						IDC_EDIT_URL,  // identifier of control
						url, // address of buffer for text
						sizeofarray(url) - 1    // maximum size of string
						) != 0) {
					USES_CONVERSION;

					if (player->m_URLName != NULL) memory_free(player->m_URLName);
					player->m_URLName = string_duplicate(W2A(url));
					if ((status = player->OpenURL(player->m_URLName)) == 0) {
						EndDialog(hDlg, LOWORD(wParam));
						LowerSip();
					} else {
						TCHAR *p = get_msg_error_string(status);
						player->ErrorBox(hDlg, TEXT("Cannot open %s: %s (error %d)"), url, p ? p : L"", status);
						if (p) memory_free(p);
					}
				}
				return TRUE;
			} else if (LOWORD(wParam) == IDCANCEL) {
				EndDialog(hDlg, LOWORD(wParam));
				LowerSip();
				return TRUE;
			}
			break;
	}
    return FALSE;
}


/*----------------------------------------------------------------------
|       OpenURL
+---------------------------------------------------------------------*/
int
PocketTV::OpenURL(const char *url)
{
	int		status;
	void	*value;
	USES_CONVERSION;

	SetSynchronousMode();
	OpenStreamURL(url);
	if (GetCommandStatus(&status, &value) == 0) {
		if (status != 0) {
			_msg_printf("%s: %s\n", A2W(url), msg_error_string(status));
			return status;
		} else {
			u_int		url_length;

			_msg_printf("%s was opened by SIH header-id = %d\n", A2W(url), (int)value);

			ActivateStreamControls();

			if (m_StreamName == NULL) memory_free(m_StreamName);
			url_length = strlen(url);
			m_StreamName = (TCHAR *)memory_alloc((url_length + 3) * sizeof(TCHAR));

			if (m_StreamName) {
				// start with two tabs to have right-aligned text
				m_StreamName[0] = L'\t';
				m_StreamName[1] = L'\t';
				wcsncpy(&m_StreamName[2], A2W(url), url_length);
				m_StreamName[url_length + 2] = 0;
			}
			if (m_ShowStatusBar) {
				InvalidateRect(m_Window, &m_StreamNameRect, TRUE);
			}
			// reset a few variables to start on a good foot...
			m_PendingSeek = -1;
			m_DisplayReady = false;
			m_LastPositionTimerTime = 0;
			return 0;
		}
	} else {
		abort();	// last command did not have MTVP_PMP_FLAG_SYNCHRONOUS!
		return -1;
	}
}

/*----------------------------------------------------------------------
| SetMute
+---------------------------------------------------------------------*/
void
PocketTV::SetMute(bool yes)
{
	Mute(yes);
	CheckMenuItem(m_Menu, IDM_OPTIONS_MUTE, (yes ? MF_CHECKED : MF_UNCHECKED));
	SendMessage(m_TrickCommandBar, TB_CHECKBUTTON , IDM_OPTIONS_MUTE, MAKELPARAM(yes, 0));
	SendMessage(m_TrickCommandBar, TB_CHANGEBITMAP, IDM_OPTIONS_MUTE, MAKELPARAM(yes ? 5 : 4, 0));
}

/*----------------------------------------------------------------------
| HandleWindowsMessage
+---------------------------------------------------------------------*/
LRESULT
PocketTV::HandleWindowsMessage(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	HDC hdc;
	int wmId, wmEvent;
	PAINTSTRUCT ps;

	switch (message) 
	{
//		case WM_HELP:
//			DialogBox(m_Instance, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)DialogAbout);
//			break;
		case WM_COMMAND:
			wmId    = LOWORD(wParam); 
			wmEvent = HIWORD(wParam); 
			// Parse the menu selections:
			switch (wmId)
			{
				case IDM_FILE_OPEN_FILE:
				    OpenFile();
					break;
				case IDM_FILE_OPEN_URL:
					DialogBox(m_Instance, (LPCTSTR)IDD_OPEN_URL, hWnd, (DLGPROC)DialogOpenURL);
					break;
//				case IDM_FILE_CLOSE:
//				    CloseFile();
//					break;
				case IDM_FILE_ABOUT:
					DialogBox(m_Instance, (LPCTSTR)IDD_ABOUTBOX, hWnd, (DLGPROC)DialogAbout);
					break;
				case IDM_CONTROLS_FFW:
					if (m_State == MTVP_STATE_PLAYING) {
						Pause();
						if (m_FastForward) {
							m_FastForward = false;
							break;
						}
					}
					{
						double frame_rate;
							
						frame_rate = m_VideoInfos.framerate;
						
//						SetFramerate(FAST_FORWARD_COEF * frame_rate);
						SetRateFactor(FAST_FORWARD_COEF);
						m_FastForward = true;
						Play();
					}
					break;
				case IDM_CONTROLS_PLAY:
					if (m_State == MTVP_STATE_PLAYING) {
						Pause();
						if (!m_FastForward) {
							break;
						}
					}
//					SetFramerate(0.0);
					SetRateFactor(1.0);
					m_FastForward = false;
					Play();
					break;
				case IDM_CONTROLS_PAUSE:
					Pause();
					break;
				case IDM_CONTROLS_STOP:
//					ShowTimePosition(0);
					Pause();
					MySeek(0);
//					Seek(0);
//					m_DisplayReady = false;
					break;
				case IDM_CONTROLS_STEP:
					Steps(1);
					GetPosition();
					break;
				case IDM_CONTROLS_REWIND:
//					ShowTimePosition(0);
					MySeek(0);
//					Seek(0);
//					m_DisplayReady = false;
					break;
				case IDM_OPTIONS_AUDIO_CHANNEL_BOTH:
					SetAudioChannel(MTVP_AUDIO_CHANNEL_BOTH);
					CheckMenuRadioItem(m_Menu, IDM_OPTIONS_AUDIO_CHANNEL_FIRST,
											  IDM_OPTIONS_AUDIO_CHANNEL_LAST,   
										  	  IDM_OPTIONS_AUDIO_CHANNEL_BOTH,
											  MF_BYCOMMAND);
					break;
				case IDM_OPTIONS_AUDIO_CHANNEL_LEFT:
					SetAudioChannel(MTVP_AUDIO_CHANNEL_LEFT);
					CheckMenuRadioItem(m_Menu, IDM_OPTIONS_AUDIO_CHANNEL_FIRST,
											  IDM_OPTIONS_AUDIO_CHANNEL_LAST,   
										  	  IDM_OPTIONS_AUDIO_CHANNEL_LEFT,
											  MF_BYCOMMAND);
					break;
				case IDM_OPTIONS_AUDIO_CHANNEL_RIGHT:
					SetAudioChannel(MTVP_AUDIO_CHANNEL_RIGHT);
					CheckMenuRadioItem(m_Menu, IDM_OPTIONS_AUDIO_CHANNEL_FIRST,
											  IDM_OPTIONS_AUDIO_CHANNEL_LAST,   
										  	  IDM_OPTIONS_AUDIO_CHANNEL_RIGHT,
											  MF_BYCOMMAND);
					break;
				case IDM_OPTIONS_AUDIO_QUALITY_LOW:
					SetAudioQuality(MTVP_AUDIO_QUALITY_LOW);
					CheckMenuRadioItem(m_Menu, IDM_OPTIONS_AUDIO_QUALITY_FIRST,
											  IDM_OPTIONS_AUDIO_QUALITY_LAST,   
										  	  IDM_OPTIONS_AUDIO_QUALITY_LOW,
											  MF_BYCOMMAND);
					break;
				case IDM_OPTIONS_AUDIO_QUALITY_MEDIUM:
					SetAudioQuality(MTVP_AUDIO_QUALITY_MEDIUM);
					CheckMenuRadioItem(m_Menu, IDM_OPTIONS_AUDIO_QUALITY_FIRST,
											  IDM_OPTIONS_AUDIO_QUALITY_LAST,   
										  	  IDM_OPTIONS_AUDIO_QUALITY_MEDIUM,
											  MF_BYCOMMAND);
					break;
				case IDM_OPTIONS_AUDIO_QUALITY_HIGH:
					SetAudioQuality(MTVP_AUDIO_QUALITY_HIGH);
					CheckMenuRadioItem(m_Menu, IDM_OPTIONS_AUDIO_QUALITY_FIRST,
											  IDM_OPTIONS_AUDIO_QUALITY_LAST,   
										  	  IDM_OPTIONS_AUDIO_QUALITY_HIGH,
											  MF_BYCOMMAND);
				case IDM_OPTIONS_VIDEO_STRETCH_AUTO:
					SetVideoStretch(VOH_WIN32_STRETCH_AUTO);
					CheckMenuRadioItem(m_Menu, IDM_OPTIONS_VIDEO_STRETCH_FIRST,
											  IDM_OPTIONS_VIDEO_STRETCH_LAST,   
										  	  IDM_OPTIONS_VIDEO_STRETCH_AUTO,
											  MF_BYCOMMAND);
					break;
				case IDM_OPTIONS_VIDEO_STRETCH_NEVER:
					SetVideoStretch(VOH_WIN32_STRETCH_NEVER);
					CheckMenuRadioItem(m_Menu, IDM_OPTIONS_VIDEO_STRETCH_FIRST,
											  IDM_OPTIONS_VIDEO_STRETCH_LAST,   
										  	  IDM_OPTIONS_VIDEO_STRETCH_NEVER,
											  MF_BYCOMMAND);
					break;
				case IDM_OPTIONS_VIDEO_STRETCH_ALWAYS:
					SetVideoStretch(VOH_WIN32_STRETCH_ALWAYS);
					CheckMenuRadioItem(m_Menu, IDM_OPTIONS_VIDEO_STRETCH_FIRST,
											  IDM_OPTIONS_VIDEO_STRETCH_LAST,   
										  	  IDM_OPTIONS_VIDEO_STRETCH_ALWAYS,
											  MF_BYCOMMAND);
					break;
				case IDM_OPTIONS_VIDEO_STRETCH_FILL:
					SetVideoStretch(VOH_WIN32_STRETCH_FILL);
					CheckMenuRadioItem(m_Menu, IDM_OPTIONS_VIDEO_STRETCH_FIRST,
											  IDM_OPTIONS_VIDEO_STRETCH_LAST,   
										  	  IDM_OPTIONS_VIDEO_STRETCH_FILL,
											  MF_BYCOMMAND);
					break;
				case IDM_OPTIONS_LOOP: {
					UINT checked = CheckMenuItem(m_Menu, IDM_OPTIONS_LOOP, MF_CHECKED);
					Loop(checked != MF_CHECKED);
					CheckMenuItem(m_Menu, IDM_OPTIONS_LOOP, (checked != MF_CHECKED ? MF_CHECKED : MF_UNCHECKED));
					}
					break;
				case IDM_OPTIONS_AUTO_PLAY: {
					UINT checked = CheckMenuItem(m_Menu, IDM_OPTIONS_AUTO_PLAY, MF_CHECKED);
					AutoPlay(checked != MF_CHECKED);
					CheckMenuItem(m_Menu, IDM_OPTIONS_AUTO_PLAY, (checked != MF_CHECKED ? MF_CHECKED : MF_UNCHECKED));
					}
					break;
				case IDM_OPTIONS_SHOW_STATUS_WINDOW: {
					UINT checked = CheckMenuItem(m_Menu, IDM_OPTIONS_SHOW_STATUS_WINDOW, MF_CHECKED);
					if (checked != MF_CHECKED) {
						m_ShowStatusBar = true;
					} else {
						m_ShowStatusBar = false;
					}
					InvalidateRect(hWnd, &m_StreamNameRect, TRUE);
					ShowWindow(m_StatusWindow, m_ShowStatusBar ? SW_SHOW : SW_HIDE);
					UpdateVideoTargetRect();
					CheckMenuItem(m_Menu, IDM_OPTIONS_SHOW_STATUS_WINDOW, (m_ShowStatusBar ? MF_CHECKED : MF_UNCHECKED));
					}
					break;
				case IDM_OPTIONS_MUTE: {
					UINT checked = CheckMenuItem(m_Menu, IDM_OPTIONS_MUTE, MF_CHECKED);
					SetMute(checked != MF_CHECKED);
					}
					break;
				default:
				   return DefWindowProc(hWnd, message, wParam, lParam);
			}
			break;
		case WM_CREATE:
			break;
		case WM_PAINT:
//			RECT rt;
			hdc = BeginPaint(hWnd, &ps);
//			GetClientRect(hWnd, &rt);
//			LoadString(Instance, IDS_HELLO, szHello, MAX_LOADSTRING);
//			DrawText(hdc, szHello, _tcslen(szHello), &rt, 
//				DT_SINGLELINE | DT_VCENTER | DT_CENTER);
			if (m_ShowStatusBar) DrawStatusText(hdc, &m_StreamNameRect, m_StreamName ? m_StreamName : TEXT(""), 0);
			EndPaint(hWnd, &ps);
			Repaint();
			break;
		case WM_SETFOCUS:
			Focus();
			break;
		case WM_NOTIFY: {
			NMHDR	*nmhdr = (NMHDR	*)lParam;
			if (nmhdr->hwndFrom == m_CommandBands) {
				if (nmhdr->code == RBN_HEIGHTCHANGE) {
					UpdateVideoTargetRect();
				} else if (nmhdr->code == RBN_LAYOUTCHANGED) {
					UpdateSlidersSize();
				}
			}
			}
			break;
		case WM_TIMER:
			if (!m_PlayerReady) {
				break;
			}

			if (wParam == ID_TIMER_POSITION) {
				SYSTEMTIME  system_time;
				LONGLONG  file_time;

				// Check the system time.  If it shows that last time we looked
				// was earlier than 2 seconds before, we can assume that the machine
				// had been "powered-off" during some time.  "powered-off" means, 
				// in the case of WinCE, that the machine was put in its 'dormant' state.
				// The problem is that we have no idea of what event caused the machine
				// to wake-up.  It could be the user pressing the power button, but it
				// could also be some time alarm or other system even that cause the system
				// to power-up by itself.
				// If PocketTV was playing a video in 'loop' mode at the time the system
				// was turned off, it would start again, possible without the user knowing
				// about it, and it could drain the entire battery!
				// so, in order to avoid this unfortunate possibility, we decide to Pause
				// the player if we detect this situation.

				if (m_DisplayReady) {
					GetSystemTime(&system_time);
					SystemTimeToFileTime(&system_time, (LPFILETIME)&file_time);

					if (m_LastPositionTimerTime != 0
								&& file_time > m_LastPositionTimerTime + (TIMER_POSITION_INTERVAL + 3000) * 10000L) {
						Pause();
//						ErrorBox(m_Window, TEXT("delta time %d\n"), (int)(file_time - m_LastPositionTimerTime));
					}
					m_LastPositionTimerTime = file_time;
				}
				// note: m_LastPositionTimerTime MUST be reset to 0 each time m_DisplayReady is set to false.

				if (!m_Seeking) {
					GetPosition();
				}
			} else if (wParam == ID_TIMER_FRAMERATE) {
				GetActualFramerate();
			} else if (wParam == ID_TIMER_KEEPALIVE) {
				KeepAlive();
			}
			break;
		case WM_HIBERNATE:
			CloseStream();
			break;
		case WM_DESTROY:
			DestroyControls();
			Quit();
			PostQuitMessage(0);
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

/*--------------------------------------------------------------------------+
| SetVideoStretch
+--------------------------------------------------------------------------*/
int
PocketTV::SetVideoStretch(VOH_WIN32_STRETCH stretch)
{
	VOH_WIN32_CNTL data;
	data.handler_id = voh_handler_id;
	data.type = VOH_WIN32_CNTL_TYPE_STRETCH;
	data.data.stretch = stretch;

	return SendCommandCustom(MTVP_COMMAND_VOH_CNTL, (const void *)&data, sizeof data);
}

/*--------------------------------------------------------------------------+
| SetVideoTargetRect
+--------------------------------------------------------------------------*/
int
PocketTV::SetVideoTargetRect(const RECT *rect) 
{
	VOH_WIN32_CNTL data;
	data.handler_id = voh_handler_id;
	data.type = VOH_WIN32_CNTL_TYPE_TARGET_RECT;
	data.data.target_rect = *rect;

	return SendCommandCustom(MTVP_COMMAND_VOH_CNTL, (const void *)&data, sizeof data);
}

/*--------------------------------------------------------------------------+
| UpdateVideoTargetRect
+--------------------------------------------------------------------------*/
void
PocketTV::UpdateVideoTargetRect()
{
	RECT rect;

	GetClientRect(m_Window, &rect);

	rect.top += CommandBands_Height(m_CommandBands);
	if (m_ShowStatusBar) {
		rect.bottom = m_StreamNameRect.top;
	}

	SetVideoTargetRect(&rect);
}

/*--------------------------------------------------------------------------+
| UpdateSliderSize
+--------------------------------------------------------------------------*/
void
PocketTV::UpdateSliderSize(HWND slider, HWND command_bar)
{
	RECT slider_rect;
	GetWindowRect(slider, &slider_rect);
	RECT commandbar_rect;
	GetWindowRect(command_bar, &commandbar_rect);
	MoveWindow(slider, slider_rect.left - commandbar_rect.left, slider_rect.top - commandbar_rect.top, 
		commandbar_rect.right - slider_rect.left, 
		slider_rect.bottom - slider_rect.top,
		TRUE);
}

/*--------------------------------------------------------------------------+
| UpdateSlidersSize
+--------------------------------------------------------------------------*/
void
PocketTV::UpdateSlidersSize()
{
	UpdateSliderSize(m_SeekSlider, m_ControlsCommandBar);
	UpdateSliderSize(m_VolumeSlider, m_TrickCommandBar);
}

/*--------------------------------------------------------------------------+
| get_time_string
+--------------------------------------------------------------------------*/
static const TCHAR *
get_time_string(
	int		sec
)
{
	static TCHAR buffer[9];

#if 0 /* Tristan Savatier - Sat Aug 23 02:57:21 PDT 1997 */
//	printf("sec = %d\n", sec);
#endif /* 0 - Tristan Savatier - Sat Aug 23 02:57:21 PDT 1997 */
	
	if (sec >= 0) {
		int		hh, mm, ss;

		hh = sec < 3600 ? 0 : sec / 3600;
		mm = (sec / 60) % 60;
		ss = sec % 60;
		
		if (hh > 99) {
			/*
			 * This can only happen if an absurd value of Video_bitrate
			 * or Mux_bitrate is received.  But if we don't test that,
			 * some bad streams can (and do) cause mtv to crash.
			 */
			swprintf(buffer, TEXT("??:??:??"));
		} else {
			swprintf(buffer, TEXT("%02d:%02d:%02d"), hh, mm, ss);
		}
		if (buffer[sizeofarray(buffer) - 1] != 0) abort_now();
		
		return buffer;
	} else {
		return TEXT("");
	}
}

/*--------------------------------------------------------------------------+
| ShowTimeTotal
+--------------------------------------------------------------------------*/
void
PocketTV::ShowTimeTotal()
{

//	printf("set_time_total, Bits_filesize = %u\n", Bits_filesize);

	if (m_Filesize == 0) {
		SendMessage(m_StatusWindow, SB_SETTEXT, 1, (LPARAM)TEXT(""));
	} else {
		int	sec;
		const TCHAR *time_str;
		
		sec = GetElapsedSeconds(m_Filesize);
	
		time_str = get_time_string(sec);

		SendMessage(m_StatusWindow, SB_SETTEXT, 1, (LPARAM)time_str);
		SendMessage(m_SeekSlider, TBM_SETRANGEMAX, TRUE, TRACK_SLIDER_MAX);
	}
//	InvalidateRect(m_StatusWindow, NULL, TRUE);
}


/*--------------------------------------------------------------------------+
| GetElapsedSeconds														|
+--------------------------------------------------------------------------*/
int
PocketTV::GetElapsedSeconds(
	u_int		pos
)
{
	if (m_PackInfos.mux_rate / BITS(char) != 0) {
		return (int)((u_int)pos / (m_PackInfos.mux_rate / BITS(char)));
	} else if (m_VideoInfos.bitrate / BITS(char) != 0) {
		return (int)((u_int)pos / (m_VideoInfos.bitrate / BITS(char)));
	} else {
		return -1;
	}
}

/*--------------------------------------------------------------------------+
| ShowTimePosition
+--------------------------------------------------------------------------*/
void
PocketTV::ShowTimePosition(u_int pos)
{
	const TCHAR *time_str;
	int	sec;

	sec = GetElapsedSeconds(pos);
	time_str = get_time_string(sec);
	
	SendMessage(m_StatusWindow, SB_SETTEXT, 0, (LPARAM)time_str);
	if (!m_Seeking && m_DisplayReady) {
		SendMessage(m_SeekSlider, TBM_SETPOS, TRUE, (LPARAM)(pos * m_TrackSliderMaxOverFilesize));
//		SendMessage(m_StatusWindow, SB_SETTEXT, 2, (LPARAM)TEXT("setpos"));
	}
}

/*--------------------------------------------------------------------------+
| ShowTimeElapsed
+--------------------------------------------------------------------------*/
void
PocketTV::ShowTimeElapsed()
{
	ShowTimePosition(m_Position);
}

/*--------------------------------------------------------------------------+
| ShowBitrate
+--------------------------------------------------------------------------*/
void
PocketTV::ShowBitrate()
{
	const TCHAR buf[20];
	int		bitrate = m_PackInfos.mux_rate ? m_PackInfos.mux_rate : m_VideoInfos.bitrate;

	if (bitrate != 0) {
		swprintf(buf, TEXT("%u Kb/s"), bitrate / 1000);

		SendMessage(m_StatusWindow, SB_SETTEXT, 3, (LPARAM)buf);
	} else {
		SendMessage(m_StatusWindow, SB_SETTEXT, 3, (LPARAM)TEXT(""));
	}
}


/*--------------------------------------------------------------------------+
| OnNotifyPackInfos
+--------------------------------------------------------------------------*/
void
PocketTV::OnNotifyPackInfos(const MTVP_PMP_DATA_PACK_INFOS *ptr)
{
	m_PackInfos = *ptr;

	ShowTimeTotal();
	ShowBitrate();
}

/*--------------------------------------------------------------------------+
| OnNotifyVideoInfos
+--------------------------------------------------------------------------*/
void
PocketTV::OnNotifyVideoInfos(const MTVP_PMP_DATA_VIDEO_INFOS *ptr)
{
	m_VideoInfos = *ptr;

	ShowTimeTotal();
	ShowBitrate();
}

/*--------------------------------------------------------------------------+
| OnNotifyFileSize
+--------------------------------------------------------------------------*/
void
PocketTV::OnNotifyFileSize(u_int size)
{
	m_Filesize = size;
	m_TrackSliderMaxOverFilesize = (double)TRACK_SLIDER_MAX / m_Filesize;
	m_FilesizeOverTrackSliderMax = (double)m_Filesize * (1.0 / TRACK_SLIDER_MAX);

	ShowTimeTotal();
}

/*--------------------------------------------------------------------------+
| ShowActualFramerate
+--------------------------------------------------------------------------*/
void
PocketTV::ShowActualFramerate()
{
	if (m_ActualFramerate == 0) {
		SendMessage(m_StatusWindow, SB_SETTEXT, 2, (LPARAM)TEXT(""));
	} else {
		const TCHAR buf[20];
		swprintf(buf, TEXT("%4.2f fps"), m_ActualFramerate);

		SendMessage(m_StatusWindow, SB_SETTEXT, 2, (LPARAM)buf);
	}
}

/*--------------------------------------------------------------------------+
| OnNotifyActualFramerate
+--------------------------------------------------------------------------*/
void
PocketTV::OnNotifyActualFramerate(double x)
{
	m_ActualFramerate = x;
	ShowActualFramerate();
}


/*--------------------------------------------------------------------------+
| OnNotifyPosition
+--------------------------------------------------------------------------*/
void
PocketTV::OnNotifyPosition(int pos)
{
	if (pos != -1) {
		m_Position = pos;
	} else {
//		SendMessage(m_StatusWindow, SB_SETTEXT, 2, (LPARAM)TEXT("pos=0"));
		m_Position = 0;
	}

	ShowTimeElapsed();
}

/*--------------------------------------------------------------------------+
| OnNotifyDisplayReady
+--------------------------------------------------------------------------*/
void
PocketTV::OnNotifyDisplayReady()
{
//	if (m_Seeking) {
//		GetPosition();
//	}

	if (m_PendingSeek >= 0) {
		Seek(m_PendingSeek);
		m_DisplayReady = false;
		m_PendingSeek = -1;
		m_LastPositionTimerTime = 0;
	} else {
		m_DisplayReady = true;
	}
}

/*--------------------------------------------------------------------------+
| OnNotifyVolume
+--------------------------------------------------------------------------*/
void
PocketTV::OnNotifyVolume(double volume)
{
	if (!m_SettingVolume) {
		SendMessage(m_VolumeSlider, TBM_SETPOS, TRUE, NINT(volume * VOLUME_SLIDER_MAX));
	}
}

/*--------------------------------------------------------------------------+
| OnNotifyCannotShowVideo
+--------------------------------------------------------------------------*/
void
PocketTV::OnNotifyCannotShowVideo(int status)
{
	TCHAR *p = get_msg_error_string(status);
	ErrorBox(m_Window, TEXT("Cannot show video: %s (error %d). Go to Setup>System>Memory and increase the available Program memory to at least 2MB"), p ? p : L"", status);
	if (p) memory_free(p);
}

/*--------------------------------------------------------------------------+
| OnNotifyCannotPlayAudio
+--------------------------------------------------------------------------*/
void
PocketTV::OnNotifyCannotPlayAudio(int status)
{
	SetMute(true);
	ErrorBox(m_Window, TEXT("Cannot play audio: audio resources probably in use by another program (error %d)"), status);
}

/*--------------------------------------------------------------------------+
| OnNotifyReady
+--------------------------------------------------------------------------*/
void
PocketTV::OnNotifyReady()
{
	m_PlayerReady = true;
}

/*--------------------------------------------------------------------------+
| MySeek
+--------------------------------------------------------------------------*/
void
PocketTV::MySeek(u_int n)
{
	off_t pos = (off_t)(n * m_FilesizeOverTrackSliderMax);

//	SendMessage(m_StatusWindow, SB_SETTEXT, 2, (LPARAM)TEXT("seek=0"));

	ShowTimePosition(pos);

	if (!m_DisplayReady) {
		m_PendingSeek = pos;
	} else {
		m_PendingSeek = -1;
		Seek(pos);
		m_DisplayReady = false;
		m_LastPositionTimerTime = 0;
	}
}

/*--------------------------------------------------------------------------+
| MySetVolume
+--------------------------------------------------------------------------*/
void
PocketTV::MySetVolume(u_int n)
{
	SetVolume((double)n / VOLUME_SLIDER_MAX);
}

/*----------------------------------------------------------------------
|       KeepAlive
+---------------------------------------------------------------------*/
void 
PocketTV::KeepAlive()
{
	if (m_State == MTVP_STATE_PLAYING) {
		keybd_event(1, 0, KEYEVENTF_SILENT, 0);
	}
}


/*----------------------------------------------------------------------
| SeekSliderProc
+---------------------------------------------------------------------*/
LRESULT 
PocketTV::SeekSliderProc(HWND window, UINT message, WPARAM wparam, LPARAM lparam)
{
	if (!m_SeekSlider) {
        return CallWindowProc((WNDPROC)m_ControlsCommandBarOriginalProc, 
                              window, message, wparam, lparam);
	}

    switch (message) {
      case WM_HSCROLL:
        switch(LOWORD(wparam)) {
          case SB_LINELEFT:
          case SB_LINERIGHT:
          case SB_PAGELEFT:
          case SB_PAGERIGHT:
            if (!m_Seeking) {
                m_Seeking = true;
				m_StateBeforeSeeking = m_State;
				Pause();
            }
            MySeek(SendMessage(m_SeekSlider,
                                     TBM_GETPOS, 0, 0));
            break;

          case SB_ENDSCROLL:
			m_PendingSeek = -1;
            Seek((off_t)((double)SendMessage(m_SeekSlider,
                                     TBM_GETPOS, 0, 0) * m_FilesizeOverTrackSliderMax));
            if (m_Seeking) {
				m_Seeking = false;
				if (m_StateBeforeSeeking == MTVP_STATE_PLAYING) {
				   Play();
				}
			}
            break;
          case SB_THUMBPOSITION: 
          case SB_THUMBTRACK:
            if (m_Seeking == false) {
                m_Seeking = true;
				m_StateBeforeSeeking = m_State;
				Pause();
            }
            MySeek(HIWORD(wparam));
			break;
        }
        break;

      default:
        return CallWindowProc((WNDPROC)m_ControlsCommandBarOriginalProc, 
                              window, message, wparam, lparam);
    }

    return 0;
}

/*----------------------------------------------------------------------
|       RealSeekSliderProc
+---------------------------------------------------------------------*/
LRESULT CALLBACK 
RealSeekSliderProc(HWND window, UINT message, WPARAM wparam, LPARAM lparam)
{
    PocketTV *player = (PocketTV *)GetWindowLong(window, GWL_USERDATA);

	return player->SeekSliderProc(window, message, wparam, lparam);
}


/*----------------------------------------------------------------------
| VolumeSliderProc
+---------------------------------------------------------------------*/
LRESULT 
PocketTV::VolumeSliderProc(HWND window, UINT message, WPARAM wparam, LPARAM lparam)
{
	if (!m_VolumeSlider) {
        return CallWindowProc((WNDPROC)m_TrickCommandBarOriginalProc, 
                              window, message, wparam, lparam);
	}
    switch (message) {
		case WM_COMMAND: {
			int wNotifyCode = HIWORD(wparam); // notification code 
			int wID = LOWORD(wparam);         // item, control, or accelerator identifier 
			HWND hwndCtl = (HWND) lparam;      // handle of control 
			if (wNotifyCode == BN_CLICKED && wID == IDM_CONTROLS_FFW) {
			} else {
			}
			return CallWindowProc((WNDPROC)m_TrickCommandBarOriginalProc, 
								  window, message, wparam, lparam);
			}
		break;
       case WM_HSCROLL:
        switch(LOWORD(wparam)) {
          case SB_LINELEFT:
          case SB_LINERIGHT:
          case SB_PAGELEFT:
          case SB_PAGERIGHT:
			  if (!m_SettingVolume) {
				  m_SettingVolume = true;
			  }
            MySetVolume(SendMessage(m_VolumeSlider,
                                     TBM_GETPOS, 0, 0));
            break;

          case SB_THUMBPOSITION: 
          case SB_ENDSCROLL:
            MySetVolume(SendMessage(m_VolumeSlider,
                                     TBM_GETPOS, 0, 0));
			  if (m_SettingVolume) {
				  m_SettingVolume = false;
			  }
            break;
          case SB_THUMBTRACK:
			  if (!m_SettingVolume) {
				  m_SettingVolume = true;
			  }
            MySetVolume(HIWORD(wparam));
			break;
        }
        break;

      default:
        return CallWindowProc((WNDPROC)m_TrickCommandBarOriginalProc, 
                              window, message, wparam, lparam);
    }

    return 0;
}

/*----------------------------------------------------------------------
|       RealVolumeSliderProc
+---------------------------------------------------------------------*/
LRESULT CALLBACK 
RealVolumeSliderProc(HWND window, UINT message, WPARAM wparam, LPARAM lparam)
{
    PocketTV *player = (PocketTV *)GetWindowLong(window, GWL_USERDATA);

	return player->VolumeSliderProc(window, message, wparam, lparam);
}

/*----------------------------------------------------------------------
| CreateSlider
+---------------------------------------------------------------------*/
HWND
PocketTV::CreateSlider(
	HWND command_bar,
	u_int	last_button_index,
	WNDPROC *command_bar_original_proc,
	WNDPROC slider_proc
)
{
	RECT commandbar_rect;
	GetWindowRect(command_bar, &commandbar_rect);
	RECT last_button_rect;
	if (last_button_index == 0) {
		last_button_rect.left = last_button_rect.right = last_button_rect.top = 0;
		last_button_rect.bottom = commandbar_rect.bottom - commandbar_rect.top;
	} else {
		SendMessage(command_bar, TB_GETITEMRECT,
					last_button_index, (LPARAM)&last_button_rect);
	}

	
    // subclass the track command bar to trap slider messages 
    SetWindowLong(command_bar, GWL_USERDATA, (LONG)this);
    *command_bar_original_proc = 
            (WNDPROC)GetWindowLong(command_bar, GWL_WNDPROC);
    SetWindowLong(command_bar, GWL_WNDPROC, (LONG)slider_proc);

    // create the track slider
    return CreateWindow( 
        TRACKBAR_CLASS, 
        NULL,
        WS_CHILD        |
        WS_VISIBLE		| 
        TBS_BOTH		|
        TBS_NOTICKS,                
        last_button_rect.right,
        last_button_rect.top,
        commandbar_rect.right - commandbar_rect.left - last_button_rect.right,
		last_button_rect.bottom - last_button_rect.top,
        command_bar,
        NULL,
        m_Instance,
        NULL);
}


/*----------------------------------------------------------------------
| InitControls
+---------------------------------------------------------------------*/
void
PocketTV::InitControls()
{
	int ret;
	INITCOMMONCONTROLSEX icex;
	icex.dwSize = sizeof(icex);
	icex.dwICC  = ICC_BAR_CLASSES | ICC_COOL_CLASSES;
	InitCommonControlsEx(&icex);

	m_CommandBands = CommandBands_Create(m_Instance, m_Window, 0,
										RBS_VARHEIGHT | RBS_BANDBORDERS, NULL);

	REBARBANDINFO info[3];

	info[0].cbSize = sizeof(REBARBANDINFO);
	info[0].fMask = RBBIM_ID | RBBIM_STYLE;
	info[0].fStyle = RBBS_NOGRIPPER;
	info[0].wID = IDBAND_MENU;

	info[1].cbSize = sizeof(REBARBANDINFO);
	info[1].fMask = RBBIM_ID | RBBIM_STYLE;
	info[1].fStyle = RBBS_BREAK;
	info[1].wID = IDBAND_CONTROLS;

	info[2].cbSize = sizeof(REBARBANDINFO);
	info[2].fMask = RBBIM_ID | RBBIM_STYLE;
	info[2].fStyle = RBBS_BREAK;
	info[2].wID = IDBAND_CONTROLS;

	CommandBands_AddBands(m_CommandBands, m_Instance, sizeofarray(info), info);

//	m_MenuCommandBar = CommandBar_Create(m_Instance, m_Window, 1);			
	m_MenuCommandBar = CommandBands_GetCommandBar(m_CommandBands, 0);			
	CommandBar_InsertMenubar(m_MenuCommandBar, m_Instance, IDM_MENU, 0);

	CommandBar_AddBitmap(m_MenuCommandBar, HINST_COMMCTRL, IDB_STD_SMALL_COLOR , 0, 0, 0);
	TBBUTTON menu_buttons [] = {
		{0, 0, 0, TBSTYLE_SEP, 0, 0, 0},
		{STD_FILEOPEN, IDM_FILE_OPEN, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0}
	};
	CommandBar_AddButtons(m_MenuCommandBar, sizeofarray(menu_buttons), menu_buttons);
	m_Menu = CommandBar_GetMenu(m_MenuCommandBar, 0); 

	m_ControlsCommandBar = CommandBands_GetCommandBar(m_CommandBands, 1);			

	m_SeekSlider = CreateSlider(m_ControlsCommandBar, 0,
							&m_ControlsCommandBarOriginalProc, RealSeekSliderProc);

	m_TrickCommandBar = CommandBands_GetCommandBar(m_CommandBands, 2);			
	CommandBar_AddBitmap(m_TrickCommandBar, m_Instance, IDB_CONTROLS, 4, 0, 0);
	TBBUTTON trick_buttons [] = {
		{7, IDM_CONTROLS_STOP, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0},
		{0, IDM_CONTROLS_PLAY, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0},
		{0, 0, 0, TBSTYLE_SEP, 0, 0, 0},
		{6, IDM_CONTROLS_FFW, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0},
		{3, IDM_CONTROLS_REWIND, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0},
		{2, IDM_CONTROLS_STEP, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0},
		{0, 0, 0, TBSTYLE_SEP, 0, 0, 0},
		{4, IDM_OPTIONS_MUTE, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0, 0, 0},
	};
	CommandBar_AddButtons(m_TrickCommandBar, sizeofarray(trick_buttons), trick_buttons);

	static const TCHAR *trick_tooltips[sizeofarray(trick_buttons)];
	trick_tooltips[0] = (const TCHAR *)MyLoadString(m_Instance, IDS_TOOLTIPS_STOP);
	trick_tooltips[1] = (const TCHAR *)MyLoadString(m_Instance, IDS_TOOLTIPS_PLAY_PAUSE);
	trick_tooltips[2] = (const TCHAR *)MyLoadString(m_Instance, IDS_TOOLTIPS_FFW_PAUSE);
	trick_tooltips[3] = (const TCHAR *)MyLoadString(m_Instance, IDS_TOOLTIPS_REWIND);
	trick_tooltips[4] = (const TCHAR *)MyLoadString(m_Instance, IDS_TOOLTIPS_STEP);
	trick_tooltips[5] = (const TCHAR *)MyLoadString(m_Instance, IDS_TOOLTIPS_MUTE);
	CommandBar_AddToolTips(m_TrickCommandBar, sizeofarray(trick_tooltips), trick_tooltips);

	m_VolumeSlider = CreateSlider(m_TrickCommandBar, sizeofarray(trick_buttons) - 1,
							&m_TrickCommandBarOriginalProc, RealVolumeSliderProc);
	SendMessage(m_VolumeSlider, TBM_SETRANGEMAX, TRUE, VOLUME_SLIDER_MAX);

	CommandBands_AddAdornments(m_CommandBands, m_Instance, 0, 0);
	CommandBands_Show(m_CommandBands, TRUE);

	// create status window
	m_StatusWindow = CreateStatusWindow(WS_CHILD|WS_VISIBLE, NULL, m_Window, ID_STATUS_WINDOW);
	int width[] = {60, 120, 180, -1};
	SendMessage(m_StatusWindow, SB_SETPARTS, sizeofarray(width), (LPARAM)width);
	m_ShowStatusBar = true;

	// determine location of  stream name rectangle (above status window)
	GetWindowRect(m_StatusWindow, &m_StreamNameRect);
	u_int height = m_StreamNameRect.bottom - m_StreamNameRect.top;
	m_StreamNameRect.top -= height;
	m_StreamNameRect.bottom -= height;

	// fire up the timers
	ret = SetTimer(m_Window, ID_TIMER_POSITION, TIMER_POSITION_INTERVAL, 0);
	ret = SetTimer(m_Window, ID_TIMER_FRAMERATE, TIMER_FRAMERATE_INTERVAL, 0);
	ret = SetTimer(m_Window, ID_TIMER_KEEPALIVE, TIMER_KEEPALIVE_INTERVAL, 0);
//	SYSTEMTIME system_time;
//	LONGLONG file_time;
//	GetLocalTime(&system_time);
//	SystemTimeToFileTime(&system_time, (LPFILETIME)&file_time);
// 	m_LastPositionTimerTime = file_time;

	DeactivateStreamControls();
	UpdateVideoTargetRect();
}

/*----------------------------------------------------------------------
| DestroyControls
+---------------------------------------------------------------------*/
void
PocketTV::DestroyControls()
{
	CommandBar_Destroy(m_MenuCommandBar);
	m_MenuCommandBar = 0;			
}

/*----------------------------------------------------------------------
| SetStreamControlsState
+---------------------------------------------------------------------*/
void
PocketTV::SetStreamControlsState(bool enabled)
{
	int	lparam = MAKELPARAM(enabled ? TBSTATE_ENABLED : TBSTATE_INDETERMINATE, 0);

	SendMessage(m_TrickCommandBar, TB_SETSTATE, IDM_CONTROLS_STOP, lparam);
	SendMessage(m_TrickCommandBar, TB_SETSTATE, IDM_CONTROLS_PLAY, lparam);
	SendMessage(m_TrickCommandBar, TB_SETSTATE, IDM_CONTROLS_REWIND, lparam);
	SendMessage(m_TrickCommandBar, TB_SETSTATE, IDM_CONTROLS_STEP, lparam);
	SendMessage(m_TrickCommandBar, TB_SETSTATE, IDM_CONTROLS_FFW, lparam);

	int	state = enabled ? MF_ENABLED : MF_GRAYED;

	EnableMenuItem(m_Menu, IDM_CONTROLS_PLAY, state);
	EnableMenuItem(m_Menu, IDM_CONTROLS_PAUSE, state);
	EnableMenuItem(m_Menu, IDM_CONTROLS_STOP, state);
	EnableMenuItem(m_Menu, IDM_CONTROLS_STEP, state);
	EnableMenuItem(m_Menu, IDM_CONTROLS_REWIND, state);
//	EnableMenuItem(m_Menu, IDM_FILE_CLOSE, state);

	// enable/disable the track slider
	EnableWindow(m_SeekSlider, enabled);
}

/*----------------------------------------------------------------------
| ActivateStreamControls
+---------------------------------------------------------------------*/
void
PocketTV::ActivateStreamControls()
{
	SetStreamControlsState(true);
}

/*----------------------------------------------------------------------
| DeactivateStreamControls
+---------------------------------------------------------------------*/
void
PocketTV::DeactivateStreamControls()
{
	SetStreamControlsState(false);
}

/*----------------------------------------------------------------------
| OnNotifyState
+---------------------------------------------------------------------*/
void
PocketTV::OnNotifyState(MTVP_STATE state)
{
	TBBUTTONINFO button;

//	_msg_printf("state=%d\n", state);

	if (m_Seeking && m_State == MTVP_STATE_PLAYING) {
		return;
	}

	m_State = state;

	button.cbSize = sizeof button;
	button.dwMask = TBIF_IMAGE;

	if (state == MTVP_STATE_PLAYING) {
		button.iImage = 1;
		SendMessage(m_TrickCommandBar, TB_SETBUTTONINFO, m_FastForward ? IDM_CONTROLS_FFW : IDM_CONTROLS_PLAY, (LPARAM)&button);
		if (m_FastForward) {
			EnableMenuItem(m_Menu, IDM_CONTROLS_FFW, MF_GRAYED);
			EnableMenuItem(m_Menu, IDM_CONTROLS_PLAY, MF_ENABLED);
		} else {
			EnableMenuItem(m_Menu, IDM_CONTROLS_FFW, MF_ENABLED);
			EnableMenuItem(m_Menu, IDM_CONTROLS_PLAY, MF_GRAYED);
		}
		EnableMenuItem(m_Menu, IDM_CONTROLS_PAUSE, MF_ENABLED);
	} else {
		button.iImage = 0;
		SendMessage(m_TrickCommandBar, TB_SETBUTTONINFO, IDM_CONTROLS_PLAY, (LPARAM)&button);
		button.iImage = 6;
		SendMessage(m_TrickCommandBar, TB_SETBUTTONINFO, IDM_CONTROLS_FFW, (LPARAM)&button);
		EnableMenuItem(m_Menu, IDM_CONTROLS_FFW, MF_ENABLED);
		EnableMenuItem(m_Menu, IDM_CONTROLS_PLAY, MF_ENABLED);
		EnableMenuItem(m_Menu, IDM_CONTROLS_PAUSE, MF_GRAYED);
		GetPosition();
		GetActualFramerate();
	}
}


//
//  FUNCTION: InitApplication()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    This function and its usage is only necessary if you want this code
//    to be compatible with Win32 systems prior to the 'RegisterClassEx'
//    function that was added to Windows 95. It is important to call this function
//    so that the application will get 'well formed' small icons associated
//    with it.
//
/*----------------------------------------------------------------------
| InitApplication
+---------------------------------------------------------------------*/
static BOOL
InitApplication(HINSTANCE hInstance, LPTSTR window_class_name)
{
	WNDCLASS	wc;

    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) PocketTVWindowProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MPEGTV));
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(BLACK_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= window_class_name;

	if (RegisterClass(&wc) == FALSE) return FALSE;
	return TRUE;
}

/*----------------------------------------------------------------------
| InitInstance
+---------------------------------------------------------------------*/
static BOOL
InitInstance(HINSTANCE hInstance, const TCHAR *window_class_name, const TCHAR *window_title, int nCmdShow)
{
	HWND	hWnd;
	PocketTV *player;

	hWnd = CreateWindow(window_class_name, window_title, WS_VISIBLE,
		0, 0, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hInstance, NULL);

	if (hWnd == 0) {
        return FALSE;
	}

    // assign an icon for the window
	{
	    HICON icon = (HICON)LoadImage(hInstance, MAKEINTRESOURCE(IDI_POCKETTV), 
                                IMAGE_ICON,
                                16, 16, 0);
        SendMessage(hWnd, WM_SETICON, FALSE, (LPARAM)icon);
    }

	/*
	 * Start the player
	 */

	{
		static const char *mtvp_argv[] = {
			"mtvp",	/* name of the player (if no / in pathname, will search in PATH) */
//			"-a0",	/* disable audio */
//			"-w",	/* trace everything */
//			"-aw12",
//			"-w11",
//			"-w12",
//			"-w4",
			NULL,	/* must be NULL-terminated */
		};

		/*
		 * First parameter of g2a_init() is a ptr to a NULL-terminated array
		 * of strings.
		 * The first string is the name of the Player ("mtvp").
		 * The following strings are possible parameters.
		 * If the first string is a full pathname (starting with "/")
		 * g2a_init() will run this program as "the Player".
		 *
		 * If the first string is not a full pathname (starting with "/")
		 * g2a_init() will look for the Player in the following places:
		 * - in the directory where argv[0] is located (if argv[0] contains a '/')
		 * - in the current directory (".")
		 * - in the PATH
		 *
		 * Typically first string is "mtvp", if the player was installed
		 * under this name.
		 *
		 * Second parameter of g2a_init() is a string that should be
		 * the argv[0] from the calling program.
		 */
	
		player = new PocketTV(mtvp_argv, hInstance, hWnd);
	}
	
	SetWindowLong(hWnd, GWL_USERDATA, (LONG)player);

	player->SetDisplayWindow((int)hWnd);

	player->InitControls();

//	ShowWindow(hWnd, nCmdShow);
	SetForegroundWindow(hWnd);
	UpdateWindow(hWnd);

	PostMessage(hWnd, WM_COMMAND, IDM_OPTIONS_AUDIO_CHANNEL_BOTH, 0);
	PostMessage(hWnd, WM_COMMAND, IDM_OPTIONS_AUDIO_QUALITY_MEDIUM, 0);
	PostMessage(hWnd, WM_COMMAND, IDM_OPTIONS_VIDEO_STRETCH_AUTO, 0);

	player->Loop(True);
	player->AutoPlay(True);

	player->MsgSelect(MTVP_MSG_SIH);
#ifdef NDEBUG
	player->OpenFile();
#else
	player->OpenURL("\\My Documents\\starwars.mpg");
#endif

	return TRUE;
}


/*----------------------------------------------------------------------
| OpenFile
+---------------------------------------------------------------------*/
void
PocketTV::OpenFile()
{
	char name[MAX_PATH];

	if (ChooseOpenFile(m_Window, name, sizeof(name)-1,
			FILE_CHOOSER_TYPE_MPEG_VIDEO, MyLoadString(m_Instance, IDS_WINDOW_CAPTION_OPEN_FILE))) {
		OpenURL(name);
	}
}


/*----------------------------------------------------------------------
| CloseFile
+---------------------------------------------------------------------*/
void
PocketTV::CloseFile()
{
	CloseStream();
	DeactivateStreamControls();
	m_Filesize = 0;
	m_Position = 0;
	memory_set(&m_VideoInfos, 0, sizeof(m_VideoInfos));
	memory_set(&m_PackInfos, 0, sizeof(m_PackInfos));
	if (m_StreamName) {
		memory_free(m_StreamName);
		m_StreamName = NULL;
	}
	ShowTimeElapsed();
	ShowTimeTotal();
	ShowBitrate();
}


/*----------------------------------------------------------------------
|       CheckForOtherInstance
+---------------------------------------------------------------------*/
static BOOL
CheckForOtherInstance(const TCHAR *class_name, const TCHAR *cmdline)
{
	CreateMutex(NULL, TRUE, class_name);
	if (GetLastError() == ERROR_ALREADY_EXISTS) {
		// there is an other instance running

		// find it's window, activate it
		HWND previous = FindWindow(class_name, NULL);
//		ShowWindow(previous, SW_SHOW);
		SetForegroundWindow(previous);
		
    	return TRUE;
	} 
	return FALSE;
}


/*----------------------------------------------------------------------
|       WinMain
+---------------------------------------------------------------------*/
int WINAPI
WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	MSG msg;
	HACCEL hAccelTable;
	const TCHAR	*window_title;			// The title bar text
	const TCHAR	*window_class_name;		// The window class name

	// Initialize global strings
	window_class_name = (const TCHAR *)LoadString(hInstance, IDC_POCKETTV, NULL, 0);
	window_title = (const TCHAR *)LoadString(hInstance, IDS_APP_TITLE, NULL, 0);

	// check if xaudio is already running
	if (CheckForOtherInstance(window_class_name, lpCmdLine)) {
		return 0;
	}

	if (!InitApplication(hInstance, window_class_name)) {
		return 0;
	}

	// Perform application initialization:
	if (!InitInstance (hInstance, window_class_name, window_title, nCmdShow)) {
		return FALSE;
	}

	hAccelTable = LoadAccelerators(hInstance, (LPCTSTR)IDC_POCKETTV);

	// Main message loop:
	while (GetMessage(&msg, NULL, 0, 0)) {
		if (!TranslateAccelerator(msg.hwnd, hAccelTable, &msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
	
	return msg.wParam;
}

// eof
