
#if !defined(_pockettv_h)
#define _pockettv_h

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "resource.h"

#include "mtvp/mtvp.h"
#include "mtvp/voh_win32_api.h"

#include "mtvp_player_win32.hpp"

/*--------------------------------------------------------------------------+
| PocketTV Class
+--------------------------------------------------------------------------*/
class PocketTV : public MTVP_Player_win32
{
private:
	HINSTANCE			m_Instance;			// The current instance
	HWND 				m_Window;
	HWND				m_CommandBands;		// The command band handle
	HWND				m_MenuCommandBar;	// The menu command bar handle
	HWND				m_ControlsCommandBar;	// The controls command bar handle
	HWND				m_TrickCommandBar;	// The controls command bar handle
	HWND				m_StatusWindow;		// The status window handle
	bool				m_ShowStatusBar;	// Show/hide status bar
	HMENU				m_Menu;				// The menu bar handle
	RECT				m_StreamNameRect;	// Rectangle for file name
	TCHAR			   *m_StreamName;		// name of opened stream
	bool				m_DisplayReady;		// turns True once first image is displayed after a seek
	int					m_PendingSeek;		// -1 or pending seek position
	HWND				m_SeekSlider;		// The track slider handle
	HWND				m_VolumeSlider;		// The volume slider handle
	MTVP_STATE			m_StateBeforeSeeking;
	MTVP_STATE			m_State;			// current state on player
	bool				m_SettingVolume;	// used by volume slider procedure
	bool				m_FastForward;		// True if playing fast forward
    WNDPROC				m_ControlsCommandBarOriginalProc;
    WNDPROC				m_TrickCommandBarOriginalProc;
	RECT				m_TargetRect;			// Rectangle for video
	double				m_FilesizeOverTrackSliderMax;
	double				m_TrackSliderMaxOverFilesize; // ratio of TrackSliderMax divided by Filesize
	bool				m_Seeking;			// True means we are currently seeking
	LONGLONG			m_LastPositionTimerTime;	// used by the position timer
	bool				m_PlayerReady;		// turns True once the player is fully initialized

	MTVP_PMP_DATA_PACK_INFOS m_PackInfos;
	MTVP_PMP_DATA_VIDEO_INFOS m_VideoInfos;
	u_int				m_Filesize;
	u_int				m_Position;
	double				m_ActualFramerate;
public:
	const char		   *m_URLName;			// name of last opened URL

public:
    PocketTV(const char **mtvp_argv, HINSTANCE instance, HWND hWnd) :
	  MTVP_Player_win32(mtvp_argv, instance),
	  m_Instance(instance),
	  m_Window(hWnd),
	  m_PendingSeek(-1)
	  {};
   ~PocketTV();

   LRESULT HandleWindowsMessage(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT SeekSliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT VolumeSliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	void InitControls();
	void DestroyControls();
	void SetStreamControlsState(bool enabled);
	void ActivateStreamControls();
	void DeactivateStreamControls();
	int OpenURL(const char *url);
	void OpenFile();
	void CloseFile();
	int SetVideoStretch(VOH_WIN32_STRETCH stretch);
	int GetElapsedSeconds(u_int pos);
	void GetPositionParameters();
	void ShowTimeTotal();
	void ShowTimeElapsed();
	void ShowTimePosition(u_int pos);
	void ShowActualFramerate();
	void ShowBitrate();
	void MySeek(u_int pos);
	void KeepAlive();
	int SetVideoTargetRect(const RECT *rect);
	void UpdateVideoTargetRect();
	void UpdateSlidersSize();
	void UpdateSliderSize(HWND slider, HWND command_bar);
	HWND CreateSlider(HWND command_bar, u_int last_button_index, WNDPROC *command_bar_original_proc, WNDPROC slider_proc);
	void MySetVolume(u_int volume);
	void ErrorBox(HWND window, const TCHAR *format, ...);
	void SetMute(bool yes);

	void OnNotifyState(MTVP_STATE state);
	void OnNotifyPackInfos(const MTVP_PMP_DATA_PACK_INFOS *ptr);
	void OnNotifyVideoInfos(const MTVP_PMP_DATA_VIDEO_INFOS *ptr);
	void OnNotifyFileSize(u_int size);
	void OnNotifyActualFramerate(double x);
	void OnNotifyVolume(double x);
	void OnNotifyPosition(int pos);
	void OnNotifyDisplayReady();
	void OnNotifyCannotShowVideo(int status);
	void OnNotifyCannotPlayAudio(int status);
	void OnNotifyReady();
};

/*--------------------------------------------------------------------------+
| Globals
+--------------------------------------------------------------------------*/
// no globals!

/*----------------------------------------------------------------------
|       prototypes
+---------------------------------------------------------------------*/

/*----------------------------------------------------------------------
|       macros
+---------------------------------------------------------------------*/
#define POCKETTV_HANDLE_WINDOW_MESSAGE(object_class,                     \
                                 handler,                          \
                                 window,                           \
						         message,                          \
						         wparam,                           \
						         lparam)                           \
{                                                                  \
	object_class *target;                                          \
    target = (object_class *)GetWindowLong(window, GWL_USERDATA);  \
    if (target) {												   \
        return target->##handler##(window,                         \
		                           message,                        \
		 	    				   wparam,                         \
								   lparam);                        \
	} else {                                                       \
		return DefWindowProc(window, message, wparam, lparam);     \
	}                                                              \
}

#endif // !defined(_pockettv_h)
